//*****************************************************************************
//  Copyright (c) 2021 Glenfly Tech Co., Ltd.. 
//  All Rights Reserved.
//
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Glenfly Tech Co., Ltd..;
//  the contents of this file may not be disclosed to third parties, copied or
//  duplicated in any form, in whole or in part, without the prior written
//  permission of Glenfly Tech Co., Ltd..
//
//  The copyright of the source code is protected by the copyright laws of the People's
//  Republic of China and the related laws promulgated by the People's Republic of China
//  and the international covenant(s) ratified by the People's Republic of China. 
//*****************************************************************************

#ifndef V4L2_VIP_CAM_MGR_H
#define V4L2_VIP_CAM_MGR_H

#include <linux/list.h>

struct cam_mgr;
struct cam_buf;

enum {
    CAM_BUF_AVAILABLE = 0,  // this flag means buffer was available to set to hardware
    CAM_BUF_USED = 1,       // this flag means buffer has been set to hardware
    CAM_BUF_BUSY = 2,       // this flag means buffer need to check or wait fence
    CAM_BUF_UNUSED = 3,     // this flag means buffer ready send to V4L2 and v4l2 send buf to mgr with this flag
};

struct buf_ops {
    int (*is_signaled)(struct cam_buf *buf);
    unsigned long long (*get_addr)(struct cam_buf *buf);
    void* (*get_cpu_vaddr)(struct cam_buf *buf);
};

struct cam_buf {
    struct list_head    node;
    struct buf_ops      *ops;
    uint64_t            handle;
    int                 flag;   //see CAM_BUF_*
};

static inline void cam_buf_init(struct cam_buf *buf, struct buf_ops *ops, uint64_t handle) {
//    INIT_LIST_HEAD(&buf->node);
    buf->ops = ops;
    buf->handle = handle;
    buf->flag = CAM_BUF_UNUSED;
}

struct cam_fmt {
    unsigned int format; //v4l2 format
    unsigned int width;
    unsigned int height;
    unsigned int fps_num;
    unsigned int fps_den;
};

struct cam_caps {
    int caps_num;
    struct cam_fmt *formats;  // caller provide memory space of formats array with caps_num elements
};

enum {
    CAM_BUF_FLAG_NONE   = 0,
    CAM_BUF_FLAG_RESOLUTION_CHANGED,
};

typedef int (*update_buffer_func)(void *data, struct cam_buf *buffer, int flag);

int mgr_init(unsigned int card_idx, unsigned int idx, struct cam_mgr **pmgr);
int mgr_deinit(struct cam_mgr *mgr);

int mgr_get_caps_num(struct cam_mgr *mgr);
int mgr_get_caps(struct cam_mgr *mgr, struct cam_caps *caps);
int mgr_set_format(struct cam_mgr *mgr, struct cam_fmt *fmt);

int mgr_set_callback(struct cam_mgr *mgr, update_buffer_func func, void *data);

int mgr_enable(struct cam_mgr *mgr);
int mgr_disable(struct cam_mgr *mgr);

int mgr_queue_buffer(struct cam_mgr *mgr, struct cam_buf *buffer);

#endif //V4L2_VIP_CAM_MGR_H
