//*****************************************************************************
//  Copyright (c) 2021 Glenfly Tech Co., Ltd.. 
//  All Rights Reserved.
//
//  This is UNPUBLISHED PROPRIETARY SOURCE CODE of Glenfly Tech Co., Ltd..;
//  the contents of this file may not be disclosed to third parties, copied or
//  duplicated in any form, in whole or in part, without the prior written
//  permission of Glenfly Tech Co., Ltd..
//
//  The copyright of the source code is protected by the copyright laws of the People's
//  Republic of China and the related laws promulgated by the People's Republic of China
//  and the international covenant(s) ratified by the People's Republic of China. 
//*****************************************************************************
/*
  * allocate zx vip buffer
 */

#include <linux/dma-buf.h>
#include <linux/module.h>

#include <linux/scatterlist.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/dma-mapping.h>
#include <media/videobuf2-memops.h>

#include "gf_vip_videobuf.h"

#ifndef VIP_BUFFER_VERSION_CODE
#define VIP_BUFFER_VERSION_CODE LINUX_VERSION_CODE
#endif

static int debug = 1;
#define dprintk(level, fmt, arg...)					      \
	do {								      \
		if (debug >= level)					      \
			pr_info("vip-v4l2: %s: " fmt, GF_FUNC_NAME(__func__), ## arg); \
	} while (0)

struct vb2_dc_buf {
	struct device			*dev;
    struct drm_gf_gem_object *obj;   //gf gpu memory
	void				*vaddr;
	unsigned long			size;
	void				*cookie;
	dma_addr_t			dma_addr;
	unsigned long			attrs;
	enum dma_data_direction		dma_dir;
	struct sg_table			*dma_sgt;
	struct frame_vector		*vec;

	/* MMAP related */
	struct vb2_vmarea_handler	handler;
	//refcount_t			refcount;
	//struct sg_table			*sgt_base;

	/* DMABUF related */
	struct dma_buf_attachment	*db_attach;
};

#if VIP_BUFFER_VERSION_CODE >= KERNEL_VERSION(5,11,0) 
static __inline__ void * vmap_dma_buf(struct dma_buf *dbuf)
{
    struct dma_buf_map map = {0};
 
    if(dbuf->ops->vmap(dbuf, &map))
    {
        dprintk(1,"dma_buf vmap failed!");
    }
    return map.vaddr_iomem;
}
#else
static __inline__ void * vmap_dma_buf(struct dma_buf *dbuf)
{
    return dbuf->ops->vmap(dbuf);
}
#endif
/*********************************************/
/*         callbacks for all buffers         */
/*********************************************/

static void *vb2_vip_cookie(void *buf_priv)
{
	struct vb2_dc_buf *buf = buf_priv;

	return buf->obj;
}

static void *vb2_vip_vaddr(void *buf_priv)
{
	struct vb2_dc_buf *buf = buf_priv;

	return buf->vaddr;
}

static unsigned int vb2_vip_num_users(void *buf_priv)
{
	struct vb2_dc_buf *buf = buf_priv;
    //printk(KERN_ERR "No op\n");
	return 0;
}

static void vb2_dc_prepare(void *buf_priv)
{

}

static void vb2_dc_finish(void *buf_priv)
{

}

/*********************************************/
/*        callbacks for MMAP buffers         */
/*********************************************/

static void vb2_dc_put(void *buf_priv)
{

}

static void *vb2_dc_alloc(struct device *dev, unsigned long attrs,
			  unsigned long size, enum dma_data_direction dma_dir,
			  gfp_t gfp_flags)
{

	return NULL;
}

static int vb2_dc_mmap(void *buf_priv, struct vm_area_struct *vma)
{

	return 0;
}


/*********************************************/
/*       callbacks for USERPTR buffers       */
/*********************************************/

static void vb2_dc_put_userptr(void *buf_priv)
{

}

static void *vb2_dc_get_userptr(struct device *dev, unsigned long vaddr,
	unsigned long size, enum dma_data_direction dma_dir)
{
	return NULL;
}

/*********************************************/
/*       callbacks for DMABUF buffers        */
/*********************************************/
static int vb2_vip_map_dmabuf(void *mem_priv)
{
	struct vb2_dc_buf *buf = mem_priv;
	struct sg_table *sgt;
	unsigned long contig_size;

	//pr_info("vb2_vip_map_dmabuf--------\n");

	return 0;
}

static void vb2_vip_unmap_dmabuf(void *mem_priv)
{
	struct vb2_dc_buf *buf = mem_priv;
	struct sg_table *sgt = buf->dma_sgt;

    //pr_info("vb2_dc_unmap_dmabuf----\n");
}

static void vb2_vip_detach_dmabuf(void *mem_priv)
{
	struct vb2_dc_buf *buf = mem_priv;
    struct drm_gf_gem_object *obj = buf->obj;
    struct dma_buf *dbuf = buf->db_attach->dmabuf;

    if (obj && obj->info.segment_id == 1) {
        dbuf->ops->vunmap(dbuf, buf->vaddr);
        buf->vaddr = NULL;
        dprintk(1, "vb2_vip_detach_dmabuf: vunmap the segment 1 buffer\n");
    }

    kfree(buf->db_attach);
    kfree(buf);

    dprintk(1, "vb2_vip_detach_dmabuf: ok\n");
}

#if VIP_BUFFER_VERSION_CODE >= KERNEL_VERSION(4,8,0)
//[4.8.0, )
static void *vb2_vip_attach_dmabuf(struct device *dev, struct dma_buf *dbuf,
	unsigned long size, enum dma_data_direction dma_dir)
{
	struct vb2_dc_buf *buf;
	struct dma_buf_attachment *dba;
    struct drm_gf_gem_object *obj = NULL;

    if (dbuf->size < size)
		return ERR_PTR(-EFAULT);

	//if (WARN_ON(!dev))
	//	return ERR_PTR(-EINVAL);

	buf = kzalloc(sizeof(*buf), GFP_KERNEL);
	if (!buf)
		return ERR_PTR(-ENOMEM);

	buf->dev = dev;

    if (dbuf->ops == &CONCAT(gf_dmabuf_ops,DRIVER_NAME))
    {
        buf->obj = dmabuf_to_gf_bo(dbuf);

        if (buf->obj->info.segment_id == 1)  //can map to cpu access
            buf->vaddr = vmap_dma_buf(dbuf);
        else
            buf->vaddr = NULL;

        dprintk(1, "get gem obj,segment id %d, gpu_virt_addr %llx, unpagable %d\n", buf->obj->info.segment_id, buf->obj->info.gpu_virt_addr,
            buf->obj->info.unpagable);
    }

	dba = kzalloc(sizeof(*dba), GFP_KERNEL);
	if (!dba)
		return ERR_PTR(-ENOMEM);

    dba->dev = dev;
    dba->dmabuf = dbuf;

	buf->dma_dir = dma_dir;
	buf->size = size;
	buf->db_attach = dba;
    dprintk(1, "vb2_vip_attach_dmabuf: ok\n");
	return buf;
}
#elif VIP_BUFFER_VERSION_CODE >= KERNEL_VERSION(4,1,0)
//[4.1.0, 4.8.0)
static void *vb2_vip_attach_dmabuf(void *alloc_ctx, struct dma_buf *dbuf,
                                          unsigned long size,
                                          enum dma_data_direction dma_dir)
{
	struct vb2_dc_buf *buf;
	struct dma_buf_attachment *dba;
    struct drm_gf_gem_object *obj = NULL;

    if (dbuf->size < size)
		return ERR_PTR(-EFAULT);

	buf = kzalloc(sizeof(*buf), GFP_KERNEL);
	if (!buf)
		return ERR_PTR(-ENOMEM);

	buf->dev = (struct device *)alloc_ctx;

    if (dbuf->ops == &CONCAT(gf_dmabuf_ops,DRIVER_NAME))
    {
        buf->obj = dmabuf_to_gf_bo(dbuf);

        if (buf->obj->info.segment_id == 1)  //can map to cpu access
            buf->vaddr = vmap_dma_buf(dbuf);
        else
            buf->vaddr = NULL;

        dprintk(1, "get gem obj,segment id %d, gpu_virt_addr %llx, unpagable %d\n", buf->obj->info.segment_id, buf->obj->info.gpu_virt_addr,
            buf->obj->info.unpagable);
    }

	dba = kzalloc(sizeof(*dba), GFP_KERNEL);
	if (!dba)
		return ERR_PTR(-ENOMEM);

    //dba->dev = dev;
    dba->dmabuf = dbuf;

	buf->dma_dir = dma_dir;
	buf->size = size;
	buf->db_attach = dba;
    dprintk(1, "vb2_vip_attach_dmabuf: ok\n");
	return buf;
}
#else
//(0, 4.1.0)
static void *vb2_vip_attach_dmabuf(void *alloc_ctx, struct dma_buf *dbuf,
				unsigned long size, int write)
{
	struct vb2_dc_buf *buf;
	struct dma_buf_attachment *dba;
    struct drm_gf_gem_object *obj = NULL;

    if (dbuf->size < size)
		return ERR_PTR(-EFAULT);

	buf = kzalloc(sizeof(*buf), GFP_KERNEL);
	if (!buf)
		return ERR_PTR(-ENOMEM);

	buf->dev = (struct device *)alloc_ctx;

    if (dbuf->ops == &CONCAT(gf_dmabuf_ops,DRIVER_NAME))
    {
        buf->obj = dmabuf_to_gf_bo(dbuf);

        if (buf->obj->info.segment_id == 1)  //can map to cpu access
            buf->vaddr = vmap_dma_buf(dbuf);
        else
            buf->vaddr = NULL;

        dprintk(1, "get gem obj,segment id %d, gpu_virt_addr %llx, unpagable %d\n", buf->obj->info.segment_id, buf->obj->info.gpu_virt_addr,
            buf->obj->info.unpagable);
    }

	dba = kzalloc(sizeof(*dba), GFP_KERNEL);
	if (!dba)
		return ERR_PTR(-ENOMEM);

    //dba->dev = dev;
    dba->dmabuf = dbuf;

	//buf->dma_dir = dma_dir;
	buf->size = size;
	buf->db_attach = dba;
    dprintk(1, "vb2_vip_attach_dmabuf: ok\n");
	return buf;
}
#endif

/*********************************************/
/*       DMA CONTIG exported functions       */
/*********************************************/

const struct vb2_mem_ops vb2_vip_memops = {
	//.alloc		= vb2_dc_alloc,
	//.put		= vb2_dc_put,
	//.get_dmabuf	= vb2_dc_get_dmabuf,
	.cookie		= vb2_vip_cookie,
	.vaddr		= vb2_vip_vaddr,
	//.mmap		= vb2_dc_mmap,
	//.get_userptr	= vb2_dc_get_userptr,
	//.put_userptr	= vb2_dc_put_userptr,
	//.prepare	= vb2_dc_prepare,
	//.finish		= vb2_dc_finish,
	.map_dmabuf	= vb2_vip_map_dmabuf,
	.unmap_dmabuf	= vb2_vip_unmap_dmabuf,
	.attach_dmabuf	= vb2_vip_attach_dmabuf, //vb2_dc_attach_dmabuf,
	.detach_dmabuf	= vb2_vip_detach_dmabuf,
	.num_users	= vb2_vip_num_users,
};
EXPORT_SYMBOL_GPL(vb2_vip_memops);


MODULE_DESCRIPTION("VIP-V4L2 memory handling routines for videobuf2");
MODULE_AUTHOR("camera");
MODULE_LICENSE("GPL");

